package text;
import java.util.*;
import java.io.IOException;
import java.io.Writer;
import java.io.StringWriter;

public class Composition implements Component {
	private class MyIterator extends NodeIterator {
		private Iterator<Component> outer = components.iterator();
		private Iterator<Node> inner = null;

		public MyIterator() {
			if (outer.hasNext()) {
				inner = outer.next().iterator();
			}
		}
		
		@Override
		public boolean hasNext() {
			if (inner==null) return false;
			while (true) {
				if (inner.hasNext()) return true;
				if (!outer.hasNext()) return false;
				inner = outer.next().iterator();
			}
		}

		@Override
		public Node next() {
			if (! hasNext()) throw new NoSuchElementException();
			return inner.next();
		}
	}
	
	private List<Component> components = new ArrayList<Component>();

	@Override
	public void add(Component c) {
		components.add(c);
	}

	@Override
	public boolean hasChildren() {
		return ! components.isEmpty();
	}

	@Override
	public Iterable<Component> getChildren() {
		return components;
	}

	@Override
	public Iterator<Node> iterator() {
		return new MyIterator();
	}

	@Override
	public String toString() {
		Writer writer = new StringWriter();
		for (Node n : this) {
			try {
				writer.write(n.toString());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return writer.toString();
	}
}